/********************************************************************************
 *   dciTBar -- Make today menubar transparent                                  *
 *                                                                              * 
 *   Copyright (C) 2004-2007 dci (dev@cpp.in)                                   *
 *                                                                              *
 *   This program is free software; you can redistribute it and/or              *
 *   modify it under the terms of the GNU General Public License version 2      *
 *   as published by the Free Software Foundation.                              *
 *                                                                              *
 *   This program is distributed in the hope that it will be useful,            *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU          *
 *   General Public License version 2 for more details.                         *
 *                                                                              *
 *   You should have received a copy of the GNU General Public License          *
 *   version 2 along with this library; if not, write to the Free Software      *
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA  *
 ********************************************************************************
 */


#include <windows.h>
#include <aygshell.h>

#define WM_DCIPING ( WM_APP + 2 )
#define IDKEY 0x696364


HANDLE hExitEvent = NULL;
BOOL bNeedBkUpdate = TRUE;

HWND hDesktop = NULL;
HWND hMenuBar = NULL;
HWND hMenuArea = NULL;
HWND hTrayArea = NULL;

WNDPROC fpDesktopProc = NULL;
WNDPROC fpMenuBarProc = NULL;
WNDPROC fpMenuAreaProc = NULL;
WNDPROC fpTrayAreaProc = NULL;


HDC hBackDC=NULL;
HBITMAP hBackBMRet=NULL;


HWND FindChildWindow ( HWND parent, wchar_t * classname )
{
	HWND hCursor = GetWindow ( parent, GW_CHILD );
	while ( hCursor )
	{
		wchar_t szClassName [ MAX_PATH ];
		GetClassName ( hCursor, szClassName, MAX_PATH );
		if ( 0 == _wcsicmp ( szClassName,classname ) ) return hCursor;
		hCursor = GetWindow ( hCursor, GW_HWNDNEXT );
	}
	return NULL;
}


/* back buffer */
void GetBackground(){
	RECT rtBar;
	int cx, cy;
	/* Get desktop window DC */
	HDC hDesktopDC = GetDC ( hDesktop );
	/* get bar sizes */
	GetClientRect ( hMenuBar, &rtBar );
	cx = rtBar.right - rtBar.left;
	cy = rtBar.bottom - rtBar.top;
	/* create cache surface */
	hBackDC = CreateCompatibleDC ( hDesktopDC );
	hBackBMRet = ( HBITMAP ) SelectObject ( hBackDC, CreateCompatibleBitmap ( hDesktopDC, cx, cy ) );
	/* convert bar coords to parent */
	MapWindowPoints ( hMenuBar, hDesktop, ( POINT * ) &rtBar, 2 );
	/* hide */
	ShowWindow(hMenuBar,SW_HIDE);
	/* force parent redraw */
	InvalidateRect ( hDesktop, &rtBar, TRUE );
	UpdateWindow ( hDesktop );
	/* save drawn pic */
	BitBlt ( hBackDC, 0, 0, cx, cy, hDesktopDC, rtBar.left, rtBar.top, SRCCOPY );
	/* show */
	ShowWindow ( hMenuBar, SW_SHOW );
	/* */
	ReleaseDC ( hDesktop, hDesktopDC );
}

void FreeBackground ( )
{
	if ( hBackDC )
	{
		DeleteObject ( SelectObject ( hBackDC, hBackBMRet ) );
		DeleteDC ( hBackDC );
		hBackBMRet = NULL;
		hBackDC = NULL;
	}
}


/* hooking procs */
long HideSIPProc ( HWND hWnd, UINT uMsg, LPARAM lParam, WPARAM wParam )
{
	if ( WM_ACTIVATE == uMsg )
	{
		HWND hSipButton = FindWindow ( L"MS_SIPBUTTON", L"MS_SIPBUTTON" );
		if ( hSipButton ) ShowWindow ( hSipButton, SW_HIDE );
	}
	else if ( ( WM_DCIPING == uMsg ) && ( IDKEY == lParam ) )
	{
		SetEvent ( hExitEvent );
		return lParam;
	}
	return CallWindowProc ( fpDesktopProc, hWnd, uMsg, lParam, wParam );
}

long TransBackProc ( HWND hWnd, UINT uMsg, LPARAM lParam, WPARAM wParam )
{
	WNDPROC fpOrigProc = NULL;
	/* */
	if ( WM_ERASEBKGND == uMsg )
	{
		if ( bNeedBkUpdate )
		{
			bNeedBkUpdate = FALSE;
			FreeBackground ( );
			GetBackground ( );			
		}		
		return TRUE;
	}
	else if ( WM_PAINT == uMsg )
	{
		RECT rtDraw, rtDrawMap;
		HDC hDrawDC = GetDC ( hWnd );
		GetUpdateRect ( hWnd, &rtDraw, FALSE );
		rtDrawMap = rtDraw;
		MapWindowPoints ( hWnd, hMenuBar, ( POINT * ) &rtDrawMap, 2 );
		BitBlt ( hDrawDC, rtDraw.left, rtDraw.top, rtDraw.right - rtDraw.left, rtDraw.bottom - rtDraw.top, hBackDC, rtDrawMap.left, rtDrawMap.top, SRCCOPY );
		ReleaseDC ( hWnd, hDrawDC );
	}
	else if ( WM_SETTINGCHANGE == uMsg )
	{
		bNeedBkUpdate=TRUE;
	}
	/* call orig */
	if ( hWnd == hMenuBar ) fpOrigProc = fpMenuBarProc;
	else if ( hWnd == hMenuArea ) fpOrigProc = fpMenuAreaProc;
	else fpOrigProc = fpTrayAreaProc;
	return CallWindowProc ( fpOrigProc, hWnd, uMsg, lParam, wParam );
}


void RedrawBar ( )
{
	InvalidateRect ( hMenuBar, NULL, TRUE );
	InvalidateRect ( hMenuArea, NULL, TRUE );
	InvalidateRect ( hTrayArea, NULL, TRUE );
	UpdateWindow ( hMenuBar );
	UpdateWindow ( hMenuArea );
	UpdateWindow ( hTrayArea );
}


/* ep */
int WINAPI WinMain ( HINSTANCE hInstance, HINSTANCE hPrevInstance, LPTSTR lpCmdLine, int nCmdShow )
{
	hDesktop = GetDesktopWindow ( );
	hMenuBar = SHFindMenuBar ( hDesktop );
	hMenuArea = FindChildWindow ( hMenuBar, L"ToolbarWindow32" );
	hTrayArea = FindChildWindow ( hMenuBar, L"Worker" );
	/* smth not found? */
	if ( hMenuArea && hTrayArea )
	{
		/* already running? */
		if ( IDKEY != SendMessage ( hDesktop, WM_DCIPING, IDKEY, IDKEY ) )
		{
			/* save current bar style */
			LONG lBarStyle=GetWindowLong ( hMenuBar, GWL_STYLE );
			/* hook all */
			fpDesktopProc = ( WNDPROC ) SetWindowLong ( hDesktop, GWL_WNDPROC, ( LONG ) &HideSIPProc );
			fpMenuBarProc = ( WNDPROC ) SetWindowLong ( hMenuBar, GWL_WNDPROC, ( LONG ) &TransBackProc );
			fpMenuAreaProc = ( WNDPROC ) SetWindowLong ( hMenuArea, GWL_WNDPROC, ( LONG ) &TransBackProc );
			fpTrayAreaProc = ( WNDPROC ) SetWindowLong ( hTrayArea, GWL_WNDPROC, ( LONG ) &TransBackProc );
			/* change style to remove the line */
			SetWindowLong ( hMenuBar, GWL_STYLE, lBarStyle & ~WS_BORDER );
			RedrawBar ( );
			/* lock */
			hExitEvent = CreateEvent ( NULL, FALSE, FALSE, NULL );
			while ( WAIT_TIMEOUT == WaitForSingleObject ( hExitEvent, INFINITE ) );
			CloseHandle ( hExitEvent );
			/* unhook */
			SetWindowLong ( hTrayArea, GWL_WNDPROC, ( LONG ) fpTrayAreaProc );
			SetWindowLong ( hMenuArea, GWL_WNDPROC, ( LONG ) fpMenuAreaProc );
			SetWindowLong ( hMenuBar, GWL_WNDPROC, ( LONG ) fpMenuBarProc );
			SetWindowLong ( hDesktop, GWL_WNDPROC, ( LONG ) fpDesktopProc);
			/* restore bar style */
			SetWindowLong ( hMenuBar, GWL_STYLE, lBarStyle );
			RedrawBar ( );
			/* */
			FreeBackground ( );
		}
	}
	else MessageBox ( NULL, L"Non PocketPC system?", L"dci", MB_ICONSTOP | MB_TOPMOST );
	/* */
	return 0;
}
